library sccomm;

import 'dart:convert';

import 'package:dson/dson.dart';

part 'typesOfSame.dart';

///Expand (decode) a json message coming to you
expandTransform(x) => new SocketMessage.fromJson(x).expand();
///shrink (encode) a object into a json message
shrinkTransform(x, [y]) => new SocketMessage(x).shrink(type: y);

class SocketMessageDecoder<String> extends Converter {
  convert(input) {
    return expandTransform(input);
  }
}

class SocketMessageEncoder extends Converter {
  convert(input) {
    return shrinkTransform(input);
  }
}

class SocketMessage {
  String jsonMessage;
  var data;
  shrink({String type}) {
    type = ((type != null) ? type : typeSerializer(data.runtimeType));
    Map jsonMessageTemp = {"type": type};
    //Start the madness!
    //Goal: turn data into a string!
    if (type == "Map" || type == "run command") {
      //type is Map
      jsonMessageTemp["data"] = new Map.fromIterables(
          (data as Map).keys.map(shrinkTransform),
          (data as Map).values.map(shrinkTransform));
    } else if (type == "List") {
      //type is List
      jsonMessageTemp["data"] = (data as List).map(shrinkTransform).toList();
    } else if (classMirrors != null &&
        (() {
          //is type in classMirrors
          for (Type tpe in classMirrors.keys) {
            if (type == tpe.toString()) return true;
          }
          return false;
        }())) {
      //serialize it using toJson()
      jsonMessageTemp["data"] = toJson(data);
    } else {
      jsonMessageTemp["data"] = JSON.encode(data);
    }

    //Madness end!
    jsonMessage = JSON.encode(jsonMessageTemp);
    return jsonMessage;
  }

  expand() {
    var dataTemp = JSON.decode(jsonMessage);
    if (dataTemp["type"] == "List") {
      //data is a List (Reverse of shrink)
      data = (dataTemp["data"] as List).map(expandTransform).toList();
    } else if (dataTemp["type"] == "Set") {
      //data is a List (Reverse of shrink)
      data = (dataTemp["data"] as Set).map(expandTransform).toSet();
    } else if (dataTemp["type"] == "Map") {
      //data is a Map (Reverse of shrink)
      data = new Map.fromIterables(
          (dataTemp["data"] as Map).keys.map(expandTransform),
          (dataTemp["data"] as Map).values.map(expandTransform));
    } else if (dataTemp["type"] == "run command") {
      //it's a servercommand, duh!
      data = dataTemp;
      return data;
    } else if (classMirrors != null &&
        () {
          //if type is in classMirrors, return true, otherwise, return false
          for (Type tpe in classMirrors.keys) {
            if (tpe.toString() == dataTemp["type"]) return true;
          }
          return false;
        }()) {
        //use fromJson
      data = fromJson(dataTemp["data"], () {
        for (Type tpe in classMirrors.keys) {
          if (tpe.toString() == dataTemp["type"]) return tpe;
        }
      }());
    } else {
      data = JSON.decode(dataTemp["data"]);
    }
    return data;
  }

  SocketMessage(this.data);
  SocketMessage.fromJson(this.jsonMessage);
  String toString() => shrink();
}
